# noinspection PyProtectedMember
from orchestrator.forms import FormPage
from orchestrator.forms.validators import Label
from orchestrator.targets import Target
from orchestrator.types import InputForm, SubscriptionLifecycle, UUIDstr
from orchestrator.types import State
from orchestrator.workflow import done, init, step, workflow
from orchestrator.workflows.steps import (
    resync,
    set_status,
    store_process_subscription,
    unsync,
)
from orchestrator.workflows.utils import wrap_modify_initial_input_form

from gso.products.product_types.iptrunk import Iptrunk
from gso.services import provisioning_proxy
from gso.services.provisioning_proxy import confirm_pp_results, \
    await_pp_results


def initial_input_form_generator(subscription_id: UUIDstr) -> InputForm:
    subscription = Iptrunk.from_subscription(subscription_id)

    class TerminateForm(FormPage):
        are_you_sure: Label = (
            f'Are you sure you want to remove {subscription.description}?'
        )

    return TerminateForm


@step('Deprovision IP trunk [DRY RUN]')
def deprovision_ip_trunk_dry(subscription: Iptrunk,
                             process_id: UUIDstr) -> State:
    provisioning_proxy.deprovision_ip_trunk(subscription, process_id)

    return {'subscription': subscription}


@step('Deprovision IP trunk [FOR REAL]')
def deprovision_ip_trunk_real(subscription: Iptrunk,
                              process_id: UUIDstr) -> State:
    provisioning_proxy.deprovision_ip_trunk(subscription, process_id, False)

    return {'subscription': subscription}


@workflow(
    'Terminate IPtrunk',
    initial_input_form=wrap_modify_initial_input_form(
        initial_input_form_generator),
    target=Target.TERMINATE,
)
def terminate_iptrunk():
    return (
            init
            >> store_process_subscription(Target.TERMINATE)
            >> unsync
            >> deprovision_ip_trunk_dry
            >> await_pp_results
            >> confirm_pp_results
            >> deprovision_ip_trunk_real
            >> await_pp_results
            >> confirm_pp_results
            >> set_status(SubscriptionLifecycle.TERMINATED)
            >> resync
            >> done
    )
